# Copyright (c) 2018-2021 Atipay, (https://www.atipay.net)

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import requests
from flask import Flask, render_template, request

app = Flask(__name__, template_folder='template')

# mock order data
order_info = {
              "desc": "Atipay Payment Python/Flask",
              "total": "50000",
              "id": "order-1",
              "customer": "Python Developer"
              }

url = "https://mipg.atipay.net/v1/"
api_key = "08106013-34f8-4cc8-8c04-2f72f225477b"


@app.route("/", methods=['GET'])
def index():
    return render_template("app.html", message='Atipay Payment With Python/Flask', order_info=order_info)


@app.route("/payment", methods=['GET'])
def payment():
    # payment urls

    token_url = url + "get-token"
    gw_url = url + "redirect-to-gateway"

    # mobile = order_info["mobile"]

    desc = order_info["desc"]
    amount = order_info["total"]
    order_id = order_info["id"]
    redirect_url = "http://www.your_domain_name.com/callback/" + order_id
    token_params = {'apiKey': api_key,
                    'redirectUrl': redirect_url,
                    'invoiceNumber': order_id,
                    'amount': amount,
                    # 'cellNumber': mobile,
                    'description': desc,
                    }

    # get token
    error_message = ""
    status = False
    r = requests.post(token_url, json=token_params)

    # check response
    if r.status_code == 200:
        response = r.json()
        if "status" in response:
            if response['status'] == "1":
                status = True
                token = response['token']
            else:
                error_message = response['errorDescription']
        else:
            error_message = response['faMessage']
    else:
        error_message = "Payment Token Failed"

    if (status):
        return render_template("payment.html", token=token, status=status, gw_url=gw_url)
    else:
        return render_template("payment.html", error_message=error_message, status=status)


@app.route("/callback/<order_id>", methods=['POST'])
def callback(order_id):
    message = ""
    status = False
    if order_id == order_info['id']:
        params = request.form
        if params['state'] == 'OK':  # will verify
            verify_url = url + "verify-payment"
            verify_params = {"apiKey": api_key,
                             "referenceNumber": params['referenceNumber']
                             }
            r = requests.post(verify_url, json=verify_params)
            if r.status_code == 200:
                response = r.json()
                order_amount = order_info['total']
                callback_amount = response['amount']
                if int(order_amount) == int(callback_amount):
                    message = "Payment is Success"
                    status = True
                else:
                    message = "Verify Failed"
            else:
                message = "Can not connect to verify url"
        else:
            message = params['state']
    else:
        message = f"Wrong order_id. expected order_id {order_info['id']}, but get order_id {order_id}"

    return render_template("callback.html", message=message, order_id=order_id, order_info=order_info, status=status)


if __name__ == '__main__':
    app.run(host='0.0.0.0', port=8080)
